/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Trivial application to use cepstral theta tts
 * 
 * Copyright (C) 2004, Andy Powell
 *
 * Andy Powell <asterisk@automated.it>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>

#include </opt/theta/include/theta.h>

static char *tdesc = "Trivial Cepstral TTS Application";

static char *app = "Cepstral";

static char *synopsis = "TTS conversion and playback";

static char *descrip = 
"Cepstral(text):  Converts text to speech using Cepstral Theta engine.\n"
"Currently only allows the use of a single voice which is the default\n"
"voice for your install. \n"
"\nUsage example:\n\nexten=> 1,1,Cepstral(\"hello world\")\n"
"\n";


STANDARD_LOCAL_USER;

LOCAL_USER_DECL;


int
mytheta_create_file(char *sayit, char *fname)
{
      theta_voice_desc *vlist, *vdesc;
      cst_voice *vox;
      int i;

      /* Call this before anything else. */
      theta_init(NULL);
      /* Get all the voices on the system. */
	//ast_log(LOG_NOTICE,"Checking voices\n");
	
	if ((vlist = theta_enum_voices(NULL, NULL)) == NULL) {
            ast_log(LOG_NOTICE,
		"No voices were found.  You may need to set the THETA_VOXPATH environment variable\n");
		return 1;
	      }

	vdesc = vlist;
	
	if ((vox = theta_load_voice(vdesc)) == NULL) 
		{
	      ast_log(LOG_NOTICE,"Failed to load voice %s\n", vdesc->voxname);
		return 1;
            }

	theta_set_outfile(vox, fname, "riff",8000, 1);
	
//	ast_log(LOG_NOTICE,"Do actual TTS\n");
	theta_tts(sayit, vox);
	
//	ast_log(LOG_NOTICE,"unload ans free vlist\n");
	
	theta_unload_voice(vox);
      theta_free_voicelist(vlist);
	
	return 0;
}

static int cepstral_exec(struct ast_channel *chan, void *data)
{
	int res = 0;
	struct localuser *u;
	char tmp[256];
	char options[25];
	int option_skip=0;
	int option_noanswer = 0;
	char *stringp;
	char sfname[255];
	char nsfname[255];
	
	if (!data || !strlen((char *)data)) {
		ast_log(LOG_WARNING, "Cepstral requires an argument (text)\n");
		return -1;
	}
	strncpy(tmp, (char *)data, sizeof(tmp)-1);
	stringp=tmp;
//trsep(&stringp, "|");
	
	LOCAL_USER_ADD(u);

//	options = strsep(&stringp, "|");
				
	ast_stopstream(chan);

      strncpy(options,"\0",sizeof(options)-1);
	
      pbx_substitute_variables_helper(chan,"${UNIQUEID}",options, sizeof(options)-1);

      strncpy(sfname,"/tmp/",sizeof(sfname)-1);
      strncat(sfname, options, strlen(options));
      strncpy(nsfname,sfname,sizeof(nsfname)-1);
		
      strncat(sfname, ".wav", 4);	
	res = mytheta_create_file(tmp,sfname);

	if(res == 0)
		{
		res = ast_streamfile(chan, nsfname, chan->language);
		if (!res) 
			res = ast_waitstream(chan, "");
		else {
			ast_log(LOG_WARNING, "ast_streamfile failed on %s for %s\n", chan->name, (char *)data);
			res = 0;
		}
		ast_stopstream(chan);
		}
	
	LOCAL_USER_REMOVE(u);
	ast_filedelete(sfname,NULL);

	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, cepstral_exec, synopsis, descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
